/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

package com.sunlabs.archsim.tests.branch;

// import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.engine.Director;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.gasp.State;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

import junit.framework.TestCase;
import junit.textui.TestRunner;

public class BranchJUTest extends TestCase {

	Director director= Director.getInstance();
	
	public BranchJUTest(String arg) {
		super(arg);
	}
	
	public static void main(String[] args){
		TestRunner.run(BranchJUTest.class);
	}
	
	public void setUp() {
	}
	
	public void tearDown(){
		director.masterClear();
	}
	
	public void testBranch_4_0_20() {
		testBranch(4,0,20);
	}
	
	public void testBranch(int branches, int start, int count) {

				
		BranchTest test= new BranchTest("branchTest", branches, start, count);
		
		// run self-check
		Boolean selfCheck= test.selfCheck();
		assertTrue(selfCheck);
		
		// run the rest of the test only if self-check passed
		if (selfCheck) {
			director.reset();
			// run for long enough to go through all the tests
			director.run();
			boolean testsPassed= test.testCheck();
			assertTrue(testsPassed);
		} // if (selfCheck)
	}
	
} // BranchhJUTest


class BranchTest extends CompositeEntity {
	
	int numberOfBranches;
	
	BranchTestSource src;
	TestBranch branch;
	BranchTestSink[] sink;
	
	int start;
	int count;
	
	public BranchTest(String name, int numBrances, int startVal, int countVal) {
		super(name);
	
		// register the number of branches, start value for the source, and the number
		// of items forthe source to generate
		numberOfBranches= numBrances;
		start = startVal;
		count= countVal;
		
		// make the source
		src= new BranchTestSource(BranchTestGlobal.SOURCE, start, count, this);
		src.setDelay(BranchTestGlobal.SOURCE_DELAY);
		
		// make the output terminal, set delays, attach it to the source
		OutputTerminal srcOut= new OutputTerminal(BranchTestGlobal.SRC_OUT, this);
		srcOut.setDelayFromActionToState(BranchTestGlobal.ACTION_TO_STATE_DELAY);
		srcOut.setDelayFromStateToAction(BranchTestGlobal.STATE_TO_ACTION_DELAY);
		// now attach
		src.attach(srcOut);
		
		// make the input terminal, set delays, attach it to the source
		InputTerminal srcAckOut= new InputTerminal(BranchTestGlobal.SRC_ACK_OUT, this);
		srcAckOut.setDelayFromActionToState(BranchTestGlobal.ACTION_TO_STATE_DELAY);
		srcAckOut.setDelayFromStateToAction(BranchTestGlobal.STATE_TO_ACTION_DELAY);
		// now attach
		src.attach(srcAckOut);
		
		// make a state: data from src to branch
		State srcDataState= new State(BranchTestGlobal.SOURCE_DATA_STATE, this);		

		// make a state: ack from src to branch
		State srcAckDataState= new State(BranchTestGlobal.ACK_SOURCE_DATA_STATE, this);		

		// Make the branch
		branch= new TestBranch(BranchTestGlobal.BRANCH,
								BranchTestGlobal.NUM_BRANCHES);
		
		// attach the terminals to the states
		srcDataState.attach(srcOut);
		srcDataState.attach(branch.getIn());
		
		srcAckDataState.attach(branch.getAckIn());
		srcAckDataState.attach(srcAckOut);
		
		// now make the sinks
		sink= new BranchTestSink[BranchTestGlobal.NUM_BRANCHES];
		State branchDataState[] = new State[BranchTestGlobal.NUM_BRANCHES];
		State ackBranchDataState[] = new State[BranchTestGlobal.NUM_BRANCHES];
		InputTerminal[] sinkIn= new InputTerminal[BranchTestGlobal.NUM_BRANCHES]; 
		OutputTerminal[] sinkAckIn= new OutputTerminal[BranchTestGlobal.NUM_BRANCHES]; 
		
		for (int i=0; i< numberOfBranches; i++ ) {
			// make the sink
			sink[i]= new BranchTestSink(BranchTestGlobal.SINK+i, this);
			sink[i].setDelay(BranchTestGlobal.SINK_DELAY);
		
			// input to the sink
			sinkIn[i]= new InputTerminal(BranchTestGlobal.SINK_IN+i);
			sinkIn[i].setDelayFromActionToState(BranchTestGlobal.ACTION_TO_STATE_DELAY);
			sinkIn[i].setDelayFromStateToAction(BranchTestGlobal.STATE_TO_ACTION_DELAY);
			// attach
			sink[i].attach(sinkIn[i]);
			
			// output from the sink
			sinkAckIn[i]= new OutputTerminal(BranchTestGlobal.SINK_ACK_IN+i, this);
			sinkAckIn[i].setDelayFromActionToState(BranchTestGlobal.ACTION_TO_STATE_DELAY);
			sinkAckIn[i].setDelayFromStateToAction(BranchTestGlobal.STATE_TO_ACTION_DELAY);
			//attach
			sink[i].attach(sinkAckIn[i]);
			
			// make the state: data from branch to sink
			branchDataState[i]= new State(BranchTestGlobal.BRANCH_DATA_STATE+i, this);
			// attach terminals
			branchDataState[i].attach(branch.getOut(i));
			branchDataState[i].attach(sinkIn[i]);
			
			// make the state: ack from sink to branch
			ackBranchDataState[i]= new State(BranchTestGlobal.ACK_BRANCH_DATA_STATE+i, this);
			// attach the terminals
			ackBranchDataState[i].attach(sinkAckIn[i]);
			ackBranchDataState[i].attach(branch.getAckOut(i));

		}
		
		// initialize active states
		srcAckDataState.setInitiallyActive(new Data(null));
		for (int i= 0; i< numberOfBranches; i++) {
			ackBranchDataState[i].setInitiallyActive(new Data(null));
		}		
		
	} // BranchTest constructor
	
	
	/** compute expected results and compare with simulated results */
	public boolean testCheck() {
		
		// compute the results, first make a vector of linked lists
		Vector<LinkedList<Data>> computedResults = new Vector<LinkedList<Data>>();
		// now make empty lists
		for (int i=0; i<numberOfBranches; i++) {
			computedResults.add(new LinkedList<Data>());
		}
		// now fill the list in a round-robin fashion
		for (int i=start; i<count; i++) {
			LinkedList<Data> l= computedResults.elementAt(i%numberOfBranches);
			l.addLast(new Data(i));
		}
		
		// prepare an array for simulated results
		Vector<LinkedList<Data>> simulatedResults= new Vector<LinkedList<Data>>();
		for (int i=0; i<numberOfBranches; i++) {
			simulatedResults.add(sink[i].getResults());
		}
		
		/*
		// Display the calculated lists
		System.out.println("Branch test calculated results");
		for (int i= 0; i<numberOfBranches; i++) {
			LinkedList<Data> cmp= computedResults.elementAt(i);
			Iterator li= cmp.iterator();
			while (li.hasNext()) {
				Object d= li.next();
				System.out.println(d);
			}
		} // for
		
		
		// Display the simulated lists
		System.out.println("\nBranch test simulated results");
		for (int i= 0; i<numberOfBranches; i++) {
			LinkedList<Data> cmp= simulatedResults.elementAt(i);
			Iterator li= cmp.iterator();
			while (li.hasNext()) {
				Object d= li.next();
				System.out.println(d);
			}
		} // for
		*/
		
		// now compare the two lists
		boolean same= true;
		for (int i= 0; (i < numberOfBranches && same); i++ ) {
			LinkedList<Data>cmp= computedResults.elementAt(i);
			LinkedList<Data>sim= simulatedResults.elementAt(i);
			same= same && cmp.equals(sim);
		}
		
		return same;
	} // testCheck
	
} // BranchTest